/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PVE_SPICE_CLIENT_PROXMOX_SERVER_H
#define PVE_SPICE_CLIENT_PROXMOX_SERVER_H

#include <QSharedPointer>

#include "proxmox-rest-client.h"
#include "proxmox-virtual-machine.h"
#include "proxmox-user.h"
#include "proxmox-node.h"
#include "errors.h"

namespace PVE {

class Domain;

class Server : public QObject
{
    Q_OBJECT
public:
    enum class Status
    {
        UP,
        DOWN
    };

    Server( const QUrl& serverURL );

    inline QUrl url() const;
    inline bool ignoreSslErrors() const;
    inline void setIgnoreSslErrors( bool flag );
    inline Status status() const;

    void login( const QString& username, const QString& password );

    void fetchNodes();
    void fetchDomains();
    void fetchStatus();

signals:
    void loginSucceeded();
    void nodesFetched( const QVector<QSharedPointer<Node>>& nodes );
    void domainsFetched( const QVector<QSharedPointer<Domain>>& domains );
    void statusFetched( Status status );
    void changed( Server* server );

    void errorOccurred( const QString& reason );

private slots:
    void onReplyFinished( QNetworkReply* reply );

private:
    QSharedPointer<ProxmoxRestClient> _restClient;

    QNetworkReply* _loginReply;
    QNetworkReply* _fetchNodesReply;
    QNetworkReply* _fetchDomains;
    QNetworkReply* _fetchStatus;
    
    QSharedPointer<User> _user;
    QVector<QSharedPointer<Node>> _nodes;
    Status _status;

    QVector<QSharedPointer<Node>>
    processFetchNodesReply( QNetworkReply* reply );

    inline void setStatus( Status status );
};

inline QUrl Server::url() const
{
    return _restClient->url();
}

inline bool Server::ignoreSslErrors() const
{
    return _restClient->ignoreSslErrors();
}

inline Server::Status Server::status() const
{
    return _status;
}

inline void Server::setStatus( Status status )
{
    if( _status == status )
    {
        return;
    }

    _status = status;

    emit changed( this );
}

inline void Server::setIgnoreSslErrors( bool flag )
{
    _restClient->setIgnoreSslErrors( flag );
}

} // namespace PVE

#endif // PVE_SPICE_CLIENT_PROXMOX_SERVER_H

