/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "account.h"

namespace PVE {

Account::Account( const QString& name,
                  const QString& user,
                  const QString& realm,
                  const QString& password,
                  const QSharedPointer<Server>& server,
                  const QVector<QSharedPointer<VMLauncher>>& launchers )
    : _name( name ),
      _user( user ),
      _realm( realm ),
      _password( password ),
      _server( server ),
      _lastLauncher(),
      _launchers( launchers )
{
    connect( server.data(),
             &Server::changed,
             this,
             &Account::onServerChanged );
}

void Account::setName( const QString& name )
{
    if( _name != name )
    {
        _name = name;
        emit changed( this );
    }
}

void Account::setUser( const QString& user )
{
    if( _user != user )
    {
        _user = user;
#ifdef USE_PASSWORD_STORAGE
        updateKeychain();
#endif
        emit changed( this );
    }
}

void Account::setRealm( const QString& realm )
{
    if( _realm != realm )
    {
        _realm = realm;
#ifdef USE_PASSWORD_STORAGE
        updateKeychain();
#endif
        emit changed( this );
    }
}

void Account::setPassword( const QString& password )
{
    if( _password != password )
    {
        _password = password;
#ifdef USE_PASSWORD_STORAGE
        updateKeychain();
#endif
        emit changed( this );
    }
}

void Account::setServer( const QSharedPointer<Server>& server )
{
    if( _server->url() != server->url() )
    {
        _server = server;
#ifdef USE_PASSWORD_STORAGE
        updateKeychain();
#endif
        emit changed( this );
    }
}

void Account::setLastLauncher( const QSharedPointer<VMLauncher>& launcher )
{
    if( !_lastLauncher || _lastLauncher != launcher )
    {
        _lastLauncher = launcher;
        emit changed( this );
    }
}

void Account::addLauncher( const QSharedPointer<VMLauncher>& launcher )
{
    _launchers.push_back( launcher );
    emit changed( this );
}

void Account::removeLauncher( const QSharedPointer<VMLauncher>& launcher )
{
    _launchers.removeAll( launcher );
    emit changed( this );
}

void Account::onServerChanged( Server* /*server*/ )
{
    emit changed( this );
}
#ifdef USE_PASSWORD_STORAGE
void Account::updateKeychain()
{
    const auto username = QString( "%1@%2" ).arg( _user ).arg( _realm );
    SyncKeychain::setUserPassword( username,
                                   _server->url().toString(),
                                   _password );
}
#endif
} // namespace PVE
