/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "connect-command.h"

#include "proxmox-node.h"
#include "proxmox-virtual-machine.h"
#include "spice-remote-viewer.h"

namespace {

const QString kRemoteViewerLaunchFailed = "Could not launch remote viewer";
const QString kVMStartFailed = "Could not start selected virtual machine";

const int kVirtualMachineStatusCheckTimeout = 1000;
const int kVMStartMaxIterations = 5;

} // namespace

namespace PVE {

ConnectCommand::ConnectCommand( QSharedPointer<Account> account,
                                int vmid,
                                bool fullscreen,
                                QObject* parent )
    : Command( parent ),
      _account( account ),
      _fullscreen( fullscreen ),
      _virtualMachine(),
      _virtualMachineStartCounter( 0 ),
      _findVMCommand( account, vmid )
{
    connect( &_findVMCommand,
             &FindVMCommand::succeeded,
             this,
             &ConnectCommand::onFindVMSucceeded );

    connect( &_findVMCommand,
             &FindVMCommand::failed,
             this,
             &ConnectCommand::failed );
}

void ConnectCommand::run()
{
    _findVMCommand.run();
}

void ConnectCommand::onFindVMSucceeded()
{
    connect( _account->server().data(),
             &Server::errorOccurred,
             this,
             &ConnectCommand::failed );

    _virtualMachine = _findVMCommand.result();
    _virtualMachineStartCounter = 0;

    connect( _virtualMachine.data(),
             &VirtualMachine::statusFetched,
             this,
             &ConnectCommand::onVMStatusFetched );

    connect( _virtualMachine.data(),
             &VirtualMachine::spiceProxyFetched,
             this,
             &ConnectCommand::onSpiceProxyFetched );

    connect( _virtualMachine.data(),
             &VirtualMachine::statusSet,
             this,
             &ConnectCommand::onVMStatusSet );

    connect( _virtualMachine.data(),
             &VirtualMachine::errorOccurred,
             this,
             &ConnectCommand::failed );

    _virtualMachine->fetchStatus();
}

void ConnectCommand::onVMStatusFetched( VirtualMachine::Status status )
{
    if( status == VirtualMachine::Status::Stopped )
    {
        if( _virtualMachineStartCounter < kVMStartMaxIterations )
        {
            ++_virtualMachineStartCounter;
            _virtualMachine->start();
        }
        else
        {
            emit failed( kVMStartFailed );
        }
    }
    else
    {
        _virtualMachine->fetchSpiceProxy();
    }
}

void ConnectCommand::onSpiceProxyFetched( const QString& spiceProxyFilePath )
{
    SpiceRemoteViewer viewer( spiceProxyFilePath );
    viewer.fullscreen( _fullscreen );

    if( !viewer.open() )
    {
        emit failed( kRemoteViewerLaunchFailed );
        return;
    }

    emit succeeded();
}

void ConnectCommand::onVMStatusSet()
{
    QTimer::singleShot( kVirtualMachineStatusCheckTimeout,
                        this,
                        &ConnectCommand::checkVirtualMachineStatus );
}

void ConnectCommand::checkVirtualMachineStatus()
{
    _virtualMachine->fetchStatus();
}

} // namespace PVE
