/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "console-application.h"

#include <iostream>

#include <QUrl>

#include "config.h"
#include "errors.h"
#include "proxmox-server.h"
#include "connect-command.h"

namespace {

const QString kNonGuiArgumentsWarn = "--user, --realm, --password, --host, "
                                     "--vmid arguments are required "
                                     "for non-gui mode";
const QString kConnectCommandWarn = "Unsupported command. "
                                    "--connect argument missing";

} // namespace

namespace PVE {

ConsoleApplication::ConsoleApplication( int& argc , char* argv[] )
    : Application( new QCoreApplication( argc, argv ) ),
      _command()
{
}

int ConsoleApplication::run()
{
    _command = processArgs();

    if( !_command )
    {
        return 1;
    }

    bind( _command.data() );

    _command->run();

    return Application::run();
}

QPointer<Command> ConsoleApplication::processArgs()
{
    QCommandLineOption connectOption( { "c", "connect" },
                                      "Connect to the virtual machine." );
    QCommandLineOption userOption( { "u", "user" },
                                     "User name at the server.",
                                     "username" );
    QCommandLineOption realmOption( { "r", "realm" },
                                      "User realm at the server(ex. PAM, PVE).",
                                      "realm" );
    QCommandLineOption passwordOption( { "p", "password" },
                                       "Password for the user.",
                                       "password" );
    QCommandLineOption hostOption( { "H", "host" },
                                   "PVE full host name.",
                                   "https://hostname:port" );
    QCommandLineOption vmidOption( { "i", "vmid" },
                                   "VMID at the server.",
                                   "vmid" );
    QCommandLineOption fullscreenOption( { "f", "full-screen" },
                                         "Open viewer with full-screen option." );
    QCommandLineOption ignoreSslErrorsOption( { "s", "ignore-ssl-errors" },
                                              "Ignore ssl errors." );
    _parser.addOptions( { connectOption,
                          userOption,
                          realmOption,
                          passwordOption,
                          hostOption,
                          vmidOption,
                          fullscreenOption,
                          ignoreSslErrorsOption } );

    if( auto command = Application::processArgs() )
    {
        return command;
    }

    if( !_parser.isSet( connectOption ) )
    {
        throw PVE::ArgumentNotFound( kConnectCommandWarn );
    }

    if( !_parser.isSet( userOption )
        || !_parser.isSet( realmOption )
        || !_parser.isSet( passwordOption )
        || !_parser.isSet( hostOption )
        || !_parser.isSet( vmidOption ) )
    {
        throw PVE::ArgumentNotFound( kNonGuiArgumentsWarn );
    }

    const auto host = QUrl::fromUserInput( _parser.value( hostOption ) );
    const auto server = QSharedPointer<Server>::create( host );
    server->setIgnoreSslErrors( _parser.isSet( ignoreSslErrorsOption ) );
    const auto user = _parser.value( userOption );
    const auto realm = _parser.value( realmOption );
    const auto password = _parser.value( passwordOption );
    const auto account = QSharedPointer<Account>::create( QString(),
                                                          user,
                                                          realm,
                                                          password,
                                                          server );
    return new ConnectCommand( account,
                               _parser.value( vmidOption ).toInt(),
                               _parser.isSet( fullscreenOption ) );
}

void ConsoleApplication::reportUnhandledException( const std::exception& exception )
{
    std::cerr << "Unhandled exception: " << exception.what() << std::endl;
}

void ConsoleApplication::reportError( const QString& reason )
{
    std::cerr << "Error occured: " << reason.toStdString() << std::endl;
}

void ConsoleApplication::bind( Command* command )
{
    connect( command,
             &Command::succeeded,
             this,
             &ConsoleApplication::onCommandSucceeded,
             Qt::QueuedConnection );

    connect( command,
             &Command::failed,
             this,
             &ConsoleApplication::onCommandFailed,
             Qt::QueuedConnection );
}

void ConsoleApplication::onCommandSucceeded()
{
    _qapp->quit();
}

void ConsoleApplication::onCommandFailed( const QString& reason )
{
    reportError( reason );
    _qapp->quit();
}

} // namespace PVE
