/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "account-dialog-controller.h"

#include "proxmox-domain.h"

namespace PVE {

AccountDialogController::AccountDialogController( QObject* parent )
    : QObject( parent ),
      _dialog(),
      _userName(),
      _password(),
      _realm()
{
}

void AccountDialogController::showDialog( const QString& title,
                                          const QString& accountName,
                                          const QString& url,
                                          const QString& userName,
                                          const QString& password,
                                          const QString& realm,
                                          bool ignoreSslErrors )
{
    _dialog = new AccountDialog;

    _dialog->setWindowTitle( title );
    _dialog->setAccountName( accountName );
    _dialog->setUserName( userName );
    _dialog->setPassword( password );
    _dialog->setRealm( realm );
    _dialog->setUrl( url );
    _dialog->setIngnoreSslErrors( ignoreSslErrors );

    connect( _dialog.data(),
             &AccountDialog::accepted,
             this,
             &AccountDialogController::accepted );

    connect( _dialog.data(),
             &AccountDialog::rejected,
             this,
             &AccountDialogController::canceled );

    connect( _dialog.data(),
             &AccountDialog::checkRequested,
             this,
             &AccountDialogController::onCheckRequested );

    _dialog->exec();
}

void AccountDialogController::onCheckRequested( const QString& url,
                                                const QString& userName,
                                                const QString& password,
                                                const QString& realm,
                                                bool ignoreSslErrors )
{
    if( url.isEmpty() )
    {
        _dialog->setUrlStatus( false );

        return;
    }

    _dialog->setBusy( true );

    _userName = userName;
    _password = password;
    _realm = realm;

    const auto server = new Server( url );
    server->setIgnoreSslErrors( ignoreSslErrors );

    connect( server,
             &Server::statusFetched,
             this,
             &AccountDialogController::onStatusFetchSucceeded );

    connect( server,
             &Server::errorOccurred,
             this,
             &AccountDialogController::onStatusFetchFailed );

    server->fetchStatus();
}

void AccountDialogController::onStatusFetchSucceeded( Server::Status status )
{
    if( status == Server::Status::DOWN )
    {
        _dialog->setBusy( false );
        _dialog->setUrlStatus( false );
        sender()->deleteLater();
        return;
    }

    _dialog->setUrlStatus( true );

    auto server = qobject_cast<Server*>( sender() );

    disconnect( server,
                &Server::errorOccurred,
                this,
                &AccountDialogController::onStatusFetchFailed );

    connect( server,
             &Server::domainsFetched,
             this,
             &AccountDialogController::onDomainsFetchSucceeded );

    connect( server,
             &Server::errorOccurred,
             this,
             &AccountDialogController::onDomainsFetchFailed );

    server->fetchDomains();
}

void AccountDialogController::onStatusFetchFailed( const QString& reason )
{
    _dialog->setBusy( false );
    emit errorOccurred( reason );
    sender()->deleteLater();
}

void AccountDialogController::onLoginSucceeded()
{
    _dialog->setBusy( false );
    _dialog->setUserNameStatus( true );
    _dialog->setPasswordStatus( true );

    sender()->deleteLater();
}

void AccountDialogController::onLoginFailed( const QString& /*reason*/ )
{
    _dialog->setBusy( false );
    _dialog->setUserNameStatus( false );
    _dialog->setPasswordStatus( false );

    sender()->deleteLater();
}

void AccountDialogController::onDomainsFetchSucceeded( const QVector<QSharedPointer<Domain>>& domains )
{
    QStringList realms;

    for( const auto& domain : domains )
    {
        realms.push_back( domain->realm() );
    }

    _dialog->setRealms( realms );

    if( !_realm.isEmpty() && !_userName.isEmpty() && !_password.isEmpty() )
    {
        if( realms.contains( _realm ) )
        {
            _dialog->setRealmStatus( true );
            _dialog->setRealm( _realm );

            auto server = qobject_cast<Server*>( sender() );
            const auto username = QString( "%1@%2" )
                                  .arg( _userName )
                                  .arg( _realm );
            disconnect( server,
                        &Server::errorOccurred,
                        this,
                        &AccountDialogController::onDomainsFetchFailed );

            connect( server,
                     &Server::loginSucceeded,
                     this,
                     &AccountDialogController::onLoginSucceeded );

            connect( server,
                     &Server::errorOccurred,
                     this,
                     &AccountDialogController::onLoginFailed );

            server->login( username, _password );
            return;
        }
        else
        {
            _dialog->setRealmStatus( false );
        }
    }

    _dialog->setBusy( false );
    sender()->deleteLater();
}

void AccountDialogController::onDomainsFetchFailed( const QString& reason )
{
    _dialog->setBusy( false );
    emit errorOccurred( reason );
    sender()->deleteLater();
}

} // namespace PVE
