/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "accounts-table-model.h"

#include "settings-manager.h"

namespace {

const int kColumnCount = 2;

const QString kNameHeader = "Name";
const QString kStatusHeader = "Status";

const QString kUp = "up";
const QString kDown = "down";

} // namespace

namespace PVE {

AccountsTableModel::AccountsTableModel( QObject* parent )
    : QAbstractTableModel( parent )
{
    const auto& accounts = SettingsManager::get().accounts();

    for( const auto& account : accounts )
    {
        connect( account.data(),
                 &Account::changed,
                 this,
                 &AccountsTableModel::onAccountChanged );
    }
}

void AccountsTableModel::addAccount( const QSharedPointer<Account>& account )
{
    beginResetModel();

    SettingsManager::get().addAccount( account );

    connect( account.data(),
             &Account::changed,
             this,
             &AccountsTableModel::onAccountChanged );

    endResetModel();
}

void AccountsTableModel::removeAccount( const QSharedPointer<Account>& account )
{
    beginResetModel();

    SettingsManager::get().removeAccount( account );

    disconnect( account.data(),
                &Account::changed,
                this,
                &AccountsTableModel::onAccountChanged );

    endResetModel();
}

int AccountsTableModel::rowCount( const QModelIndex& /*index*/ ) const
{
    return SettingsManager::get().accounts().size();
}

int AccountsTableModel::columnCount( const QModelIndex& /*index*/ ) const
{
    return kColumnCount;
}

QVariant AccountsTableModel::data( const QModelIndex& index, int role ) const
{
    const auto& accounts = SettingsManager::get().accounts();

    if( !index.isValid()
        || ( index.row() >= accounts.size() || index.row() < 0 ) )
    {
        return QVariant();
    }

    if( role == Qt::DisplayRole || role == Qt::EditRole )
    {
        const auto& account = accounts[ index.row() ];

        switch( index.column() )
        {
            case 0: return account->name();
            case 1: return account->server()->status() == Server::Status::UP ? kUp
                                                                  : kDown;
        default: break;
        }
    }

    if( role == Qt::TextAlignmentRole )
    {
        return Qt::AlignCenter;
    }

    return QVariant();
}

QVariant AccountsTableModel::headerData( int section,
                                         Qt::Orientation orientation,
                                         int role ) const
{
    if( role != Qt::DisplayRole && role != Qt::EditRole )
    {
        return QVariant();
    }

    if( orientation == Qt::Horizontal )
    {
        switch( section )
        {
            case 0: return kNameHeader;
            case 1: return kStatusHeader;

        default: break;
        }
    }

    return QVariant();
}

void AccountsTableModel::onAccountChanged( Account* account )
{
    const auto& accounts = SettingsManager::get().accounts();
    const auto size = accounts.size();

    for( int i = 0; i < size; ++i )
    {
        if( accounts[i].data() == account )
        {
            const auto index = createIndex( i, 1 );

            emit dataChanged( index, index );
            return;
        }
    }
}

} // namespace PVE
