/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "accounts-view-controller.h"

#include <QMessageBox>

#include "settings-manager.h"

namespace {

const QString kAddAccountDialogTitle = "Add account";
const QString kEditAccountDialogTitle = "Edit account";
const QString kPasswordRequestDialogTitle = "Password required";

const QString kInfoTitle = "Info";
const QString kWarningTitle = "Warning";

const QString kNoAccounts = "There is no accounts";
const QString kAccountNotSelected = "Account is not selected";
const QString kRemoveAccFormat = "Remove account %1 ?";

const int kIndexAfterRemoving = -1;

} // namespace

namespace PVE {

AccountsViewController::AccountsViewController()
    : _view( Q_NULLPTR ),
      _model(),
      _dialogController(),
      _selectedAccountIndex( 0 ),
      _timer()
{
    _timer.setInterval( SettingsManager::get().accountsUpdatingInterval() );

    connect( &_timer,
             &QTimer::timeout,
             this,
             &AccountsViewController::onTimeout );

    connect( &_dialogController,
             &AccountDialogController::errorOccurred,
             this,
             &AccountsViewController::errorOccurred );
}

void AccountsViewController::setView( AccountsView* view )
{
    _view = view;

    if( !_view )
    {
        return;
    }

    connect( _view,
             &AccountsView::addButtonClicked,
             this,
             &AccountsViewController::onAddButtonClicked );

    connect( _view,
             &AccountsView::editButtonClicked,
             this,
             &AccountsViewController::onEditButtonClicked );

    connect( _view,
             &AccountsView::removeButtonClicked,
             this,
             &AccountsViewController::onRemoveButtonClicked );

    connect( _view,
             &AccountsView::accountSelectedAtIndex,
             this,
             &AccountsViewController::onAccountSelected );

    const auto& manager = SettingsManager::get();

    const auto accounts = manager.accounts();
    const auto lastAccount = manager.lastAccount();

    _view->setModel( &_model );
    _view->select( lastAccount ? accounts.indexOf( lastAccount )
                               : 0 );
    onTimeout();
    _timer.start();
}

void AccountsViewController::onAddButtonClicked()
{
    bindAddDialog( &_dialogController );
    _dialogController.showDialog( kAddAccountDialogTitle );
}

void AccountsViewController::onEditButtonClicked()
{
    const auto account = currentAccount();

    if( !account )
    {
        return;
    }

    const auto server = account->server();

    bindEditDialog( &_dialogController );
    _dialogController.showDialog( kEditAccountDialogTitle,
                                  account->name(),
                                  server->url().toString(),
                                  account->user(),
                                  account->password(),
                                  account->realm(),
                                  server->ignoreSslErrors() );
}

void AccountsViewController::onRemoveButtonClicked()
{
    const auto account = currentAccount();

    if( !account )
    {
        return;
    }

    const auto result = QMessageBox::warning( _view,
                                              kWarningTitle,
                                              QString( kRemoveAccFormat )
                                              .arg( account->name() ),
                                              QMessageBox::Yes
                                              | QMessageBox::No );
    if( result == QMessageBox::No )
    {
        return;
    }

    _model.removeAccount( account );

    onAccountSelected( kIndexAfterRemoving );
}

void AccountsViewController::onAccountSelected( int index )
{
    _selectedAccountIndex = index;

    const auto account = index == kIndexAfterRemoving ? Q_NULLPTR
                                                      : currentAccount();
    if( account && account->password().isEmpty() )
    {
        const auto server = account->server();

        bindPasswordRequestDialog( &_dialogController );
        _dialogController.showDialog( kPasswordRequestDialogTitle,
                                      account->name(),
                                      server->url().toString(),
                                      account->user(),
                                      QString(),
                                      account->realm(),
                                      server->ignoreSslErrors() );
        return;
    }

    SettingsManager::get().setLastAccount( account );

    emit accountSwitched( account );
}

void AccountsViewController::onAddAccepted( const QString& accountName,
                                            const QString& host,
                                            const QString& userName,
                                            const QString& password,
                                            const QString& realm,
                                            bool ignoreSslErrors )
{
    onAddCanceled();

    const auto url = QUrl::fromUserInput( host );
    const auto server = QSharedPointer<Server>::create( url );
    server->setIgnoreSslErrors( ignoreSslErrors );
    const auto account = QSharedPointer<Account>::create( accountName,
                                                          userName,
                                                          realm,
                                                          password,
                                                          server );
    _model.addAccount( account );
    server->fetchStatus();
}

void AccountsViewController::onEditAccepted( const QString& accountName,
                                             const QString& host,
                                             const QString& userName,
                                             const QString& password,
                                             const QString& realm,
                                             bool ignoreSslErrors )
{
    onEditCanceled();

    const auto account = currentAccount();

    if( !account )
    {
        return;
    }

    account->setName( accountName );
    account->setUser( userName );
    account->setPassword( password );
    account->setRealm( realm );
    account->setServer( QSharedPointer<Server>::create( host ) );
    account->server()->setIgnoreSslErrors( ignoreSslErrors );
}

void AccountsViewController::onTimeout()
{
    const auto accounts = _model.accounts();

    for( const auto& account : accounts )
    {
        account->server()->fetchStatus();
    }
}

void AccountsViewController::onAddCanceled()
{
    unbindAddDialog( &_dialogController );
}

void AccountsViewController::onEditCanceled()
{
    unbindEditDialog( &_dialogController );
}

void AccountsViewController::onPasswordRequestAccepted( const QString& accountName,
                                                        const QString& host,
                                                        const QString& userName,
                                                        const QString& password,
                                                        const QString& realm,
                                                        bool ignoreSslErrors )
{
    unbindPasswordRequestDialog( &_dialogController );

    onEditAccepted( accountName,
                    host,
                    userName,
                    password,
                    realm,
                    ignoreSslErrors );

    const auto account = currentAccount();

    if( !account )
    {
        return;
    }

    SettingsManager::get().setLastAccount( account );

    emit accountSwitched( account );
}

void AccountsViewController::onPasswordRequestCanceled()
{
    unbindPasswordRequestDialog( &_dialogController );
    onAccountSelected( kIndexAfterRemoving );
}

void AccountsViewController::bindAddDialog( AccountDialogController* dialogController )
{
    connect( dialogController,
             &AccountDialogController::accepted,
             this,
             &AccountsViewController::onAddAccepted );

    connect( dialogController,
             &AccountDialogController::canceled,
             this,
             &AccountsViewController::onAddCanceled );
}

void AccountsViewController::bindEditDialog( AccountDialogController* dialogController )
{
    connect( dialogController,
             &AccountDialogController::accepted,
             this,
             &AccountsViewController::onEditAccepted );

    connect( dialogController,
             &AccountDialogController::canceled,
             this,
             &AccountsViewController::onEditCanceled );
}

void AccountsViewController::bindPasswordRequestDialog( AccountDialogController* dialogController )
{
    connect( dialogController,
             &AccountDialogController::accepted,
             this,
             &AccountsViewController::onPasswordRequestAccepted );

    connect( dialogController,
             &AccountDialogController::canceled,
             this,
             &AccountsViewController::onPasswordRequestCanceled );
}

void AccountsViewController::unbindAddDialog( AccountDialogController* dialogController )
{
    disconnect( dialogController,
                &AccountDialogController::accepted,
                this,
                &AccountsViewController::onAddAccepted );

    disconnect( dialogController,
                &AccountDialogController::canceled,
                this,
                &AccountsViewController::onAddCanceled );
}

void AccountsViewController::unbindEditDialog( AccountDialogController* dialogController )
{
    disconnect( dialogController,
                &AccountDialogController::accepted,
                this,
                &AccountsViewController::onEditAccepted );

    disconnect( dialogController,
                &AccountDialogController::canceled,
                this,
                &AccountsViewController::onEditCanceled );
}

void AccountsViewController::unbindPasswordRequestDialog( AccountDialogController* dialogController )
{
    disconnect( dialogController,
                &AccountDialogController::accepted,
                this,
                &AccountsViewController::onPasswordRequestAccepted );

    disconnect( dialogController,
                &AccountDialogController::canceled,
                this,
                &AccountsViewController::onPasswordRequestCanceled );
}

void AccountsViewController::showInfo( const QString& info )
{
    QMessageBox::information( _view, kInfoTitle, info );
}

QSharedPointer<Account> AccountsViewController::currentAccount()
{
    const auto& accounts = _model.accounts();
    const auto size = accounts.size();

    if( !size )
    {
        showInfo( kNoAccounts );
        return Q_NULLPTR;
    }

    if( _selectedAccountIndex < 0 || _selectedAccountIndex >= size )
    {
        showInfo( kAccountNotSelected );
        return Q_NULLPTR;
    }

    return accounts[_selectedAccountIndex];
}

} // namespace PVE
