/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "vm-launchers-table-model.h"

#include "global.h"

namespace {

const int kColumnCount = 3;

const QString kIDHeader = "Id";
const QString kNameHeader = "Name";
const QString kStatusHeader = "Status";

const QString kStatusNotFound = "not found";

} // namespace

namespace PVE {

VMLaunchersTableModel::VMLaunchersTableModel( const QSharedPointer<Account>& account,
                                              QObject* parent )
    : QAbstractTableModel( parent ),
      _account( account )
{
    const auto& launchers = _account->launchers();

    for( const auto& launcher : launchers )
    {
        connect( launcher.data(),
                 &VMLauncher::changed,
                 this,
                 &VMLaunchersTableModel::onLauncherChanged );
    }
}

void VMLaunchersTableModel::addVMLauncher( const QSharedPointer<VMLauncher>& launcher )
{
    beginResetModel();

    _account->addLauncher( launcher );

    connect( launcher.data(),
             &VMLauncher::changed,
             this,
             &VMLaunchersTableModel::onLauncherChanged );

    endResetModel();
}

void VMLaunchersTableModel::removeVMLauncher( const QSharedPointer<VMLauncher>& launcher )
{
    beginResetModel();

    _account->removeLauncher( launcher );

    disconnect( launcher.data(),
                &VMLauncher::changed,
                this,
                &VMLaunchersTableModel::onLauncherChanged );

    endResetModel();
}

int VMLaunchersTableModel::rowCount( const QModelIndex& /*index*/ ) const
{
    return _account->launchers().size();
}

int VMLaunchersTableModel::columnCount( const QModelIndex& /*index*/ ) const
{
    return kColumnCount;
}

QVariant VMLaunchersTableModel::data( const QModelIndex& index, int role ) const
{
    const auto& launchers = _account->launchers();

    if( !index.isValid()
        || ( index.row() >= launchers.size() || index.row() < 0 ) )
    {
        return QVariant();
    }

    if( role == Qt::DisplayRole || role == Qt::EditRole )
    {
        const auto& launcher = launchers.at( index.row() );

        switch( index.column() )
        {
            case 0: return QString::number( launcher->vmid() );
            case 1: return launcher->name();
            case 2:
            {
                if( launcher->isValid() )
                {
                    return Converter::virtualMachineStatusToString( launcher->cachedVM()->status() );
                }
                else
                {
                    return kStatusNotFound;
                }
            }

        default: break;
        }
    }

    if( role == Qt::TextAlignmentRole )
    {
        return Qt::AlignCenter;
    }

    return QVariant();
}

QVariant VMLaunchersTableModel::headerData( int section,
                                                Qt::Orientation orientation,
                                                int role ) const
{
    if( role != Qt::DisplayRole && role != Qt::EditRole )
    {
        return QVariant();
    }

    if( orientation == Qt::Horizontal )
    {
        switch( section )
        {
            case 0: return kIDHeader;
            case 1: return kNameHeader;
            case 2: return kStatusHeader;

            default:
                break;
        }
    }

    return QVariant();
}

void VMLaunchersTableModel::onLauncherChanged( VMLauncher* launcher )
{
    const auto& launchers = _account->launchers();
    const auto size = launchers.size();

    for( int i = 0; i < size; ++i )
    {
        if( launchers[i].data() == launcher )
        {
            const auto index = createIndex( i, 2 );

            emit dataChanged( index, index );
            return;
        }
    }
}

} // namespace PVE
