/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "vm-launchers-view-controller.h"

#include <QMessageBox>

#include "settings-manager.h"

namespace {

const QString kAddVMDialogTitle = "Add launcher";
const QString kEditVMDialogTitle = "Edit launcher";

const QString kInfoTitle = "Info";
const QString kWarningTitle = "Warning";
const QString kAccountNotSelected = "Account is not selected";
const QString kNoLaunchers = "There is no launchers";
const QString kLauncherNotSelected = "Launcher is not selected";
const QString kRemoveLauncherFormat = "Remove launcher %1 ?";
const QString kVMExistsFormat = "Virtual machine with id %1 already exists";

const int kIndexAfterRemoving = -1;

} // namespace

namespace PVE {

VMLaunchersViewController::VMLaunchersViewController()
    : _view( Q_NULLPTR ),
      _selectedLauncherIndex( 0 ),
      _timer(),
      _model(),
      _dialogController(),
      _launcherConnector(),
      _launcherValidator(),
      _launcherAppender()
{
    _timer.setInterval( SettingsManager::get().launchersUpdatingInterval() );

    connect( &_timer,
             &QTimer::timeout,
             this,
             &VMLaunchersViewController::onTimeout );
}

void VMLaunchersViewController::setView( VMLaunchersView* view )
{
    _view = view;

    if( !_view )
    {
        return;
    }

    connect( _view,
             &VMLaunchersView::addButtonClicked,
             this,
             &VMLaunchersViewController::onAddButtonClicked );

    connect( _view,
             &VMLaunchersView::editButtonClicked,
             this,
             &VMLaunchersViewController::onEditButtonClicked );

    connect( _view,
             &VMLaunchersView::removeButtonClicked,
             this,
             &VMLaunchersViewController::onRemoveButtonClicked );

    connect( _view,
             &VMLaunchersView::connectButtonClicked,
             this,
             &VMLaunchersViewController::onConnectButtonClicked );

    connect( _view,
             &VMLaunchersView::launcherSelectedAtIndex,
             this,
             &VMLaunchersViewController::onLauncherSelected );
}

void VMLaunchersViewController::setAccount( const QSharedPointer<Account>& account )
{
    if( !account )
    {
        _model = Q_NULLPTR;
        _dialogController = Q_NULLPTR;
        _launcherConnector = Q_NULLPTR;
        _launcherValidator = Q_NULLPTR;
        _launcherAppender = Q_NULLPTR;

        _view->setModel( Q_NULLPTR );
    }
    else
    {
        _model = new VMLaunchersTableModel( account );

        _dialogController = new VMLauncherDialogController( account );

        connect( _dialogController.data(),
                 &VMLauncherDialogController::errorOccurred,
                 this,
                 &VMLaunchersViewController::onErrorOccurred );

        _launcherConnector = new VMLauncherConnector( account );
        bind( _launcherConnector.data() );

        _launcherValidator = new VMLauncherValidator( account );
        bind( _launcherValidator.data() );

        _launcherAppender = new VMLauncherAppender( account );
        bind( _launcherAppender.data() );

        _view->setModel( _model.data() );

        const auto lastLauncher = _model->lastLauncher();

        _view->select( lastLauncher ? _model->launchers().indexOf( lastLauncher )
                                    : 0 );

        _launcherValidator->validate( _model->launchers() );

        onTimeout();
        _timer.start();
    }
}

void VMLaunchersViewController::onAddButtonClicked()
{
    if( !_dialogController )
    {
        showInfo( kAccountNotSelected );
        return;
    }

    bindAddDialog( _dialogController.data() );
    _dialogController->showDialog( kAddVMDialogTitle );
}

void VMLaunchersViewController::onEditButtonClicked()
{
    const auto launcher = currentLauncher();

    if( !launcher )
    {
        return;
    }

    bindEditDialog( _dialogController.data() );
    _dialogController->showDialog( kEditVMDialogTitle,
                                   launcher->name(),
                                   QString::number( launcher->vmid() ),
                                   launcher->arguments() );
}

void VMLaunchersViewController::onRemoveButtonClicked()
{
    const auto launcher = currentLauncher();

    if( !launcher )
    {
        return;
    }

    const auto result = QMessageBox::warning( _view,
                                              kWarningTitle,
                                              QString( kRemoveLauncherFormat )
                                              .arg( launcher->name() ),
                                              QMessageBox::Yes
                                              | QMessageBox::No );
    if( result == QMessageBox::No )
    {
        return;
    }

    _model->removeVMLauncher( launcher );

    onLauncherSelected( kIndexAfterRemoving );
}

void VMLaunchersViewController::onConnectButtonClicked()
{
    const auto launcher = currentLauncher();

    if( !launcher )
    {
        return;
    }

    _view->setBusy( true );

    _launcherConnector->connect( launcher );
}

void VMLaunchersViewController::onLauncherSelected( int index )
{
    _selectedLauncherIndex = index;

    const auto launcher = index == kIndexAfterRemoving ? Q_NULLPTR
                                                       : currentLauncher();
    if( _model )
    {
        _model->setLastLauncher( launcher );
    }
}

void VMLaunchersViewController::onAddAccepted( const QString& launcherName,
                                               int vmid,
                                               bool fullscreen )
{
    _view->setBusy( true );

    onAddCanceled();

    const auto& launchers = _model->launchers();

    for( const auto& launcher : launchers )
    {
        if( launcher->vmid() == vmid )
        {
            emit errorOccurred( QString( kVMExistsFormat )
                               .arg(  QString::number( vmid ) ) );
            _view->setBusy( false );
            sender()->deleteLater();
            return;
        }
    }

    _launcherAppender->append( vmid, launcherName, { fullscreen } );
}

void VMLaunchersViewController::onAddCanceled()
{
    unbindAddDialog( _dialogController.data() );
}

void VMLaunchersViewController::onEditAccepted( const QString& launcherName,
                                                int vmid,
                                                bool fullscreen )
{
    onEditCanceled();

    const auto launcher = currentLauncher();

    if( !launcher )
    {
        return;
    }

    launcher->setName( launcherName );
    launcher->setVmid( vmid );
    launcher->setArguments( { fullscreen } );

    if( !launcher->isValid() )
    {
        _launcherValidator->validate( launcher );
    }
}

void VMLaunchersViewController::onEditCanceled()
{
    unbindEditDialog( _dialogController.data() );
}

void VMLaunchersViewController::onTimeout()
{
    if( !_model )
    {
        return;
    }

    const auto& launchers = _model->launchers();

    for( const auto& launcher : launchers )
    {
        if( launcher->isValid() )
        {
            launcher->cachedVM()->fetchStatus();
        }
    }
}

void VMLaunchersViewController::onConnected()
{
    _view->setBusy( false );
}

void VMLaunchersViewController::onAppended( const QSharedPointer<VMLauncher>& launcher )
{
    if( _model )
    {
        _model->addVMLauncher( launcher );
    }

    _view->setBusy( false );
}

void VMLaunchersViewController::onErrorOccurred( const QString& reason )
{
    _view->setBusy( false );
    emit errorOccurred( reason );
}

void VMLaunchersViewController::bind( VMLauncherConnector* connector )
{
    connect( connector,
             &VMLauncherConnector::connected,
             this,
             &VMLaunchersViewController::onConnected );
    connect( connector,
             &VMLauncherConnector::errorOccurred,
             this,
             &VMLaunchersViewController::onErrorOccurred );
}

void VMLaunchersViewController::bind( VMLauncherValidator* validator )
{
    connect( validator,
             &VMLauncherValidator::errorOccurred,
             this,
             &VMLaunchersViewController::onErrorOccurred );
}

void VMLaunchersViewController::bind( VMLauncherAppender* appender )
{
    connect( appender,
             &VMLauncherAppender::appended,
             this,
             &VMLaunchersViewController::onAppended );
    connect( appender,
             &VMLauncherAppender::errorOccurred,
             this,
             &VMLaunchersViewController::onErrorOccurred );
}

void VMLaunchersViewController::bindAddDialog( VMLauncherDialogController* dialogController )
{
    connect( dialogController,
             &VMLauncherDialogController::accepted,
             this,
             &VMLaunchersViewController::onAddAccepted );

    connect( dialogController,
             &VMLauncherDialogController::canceled,
             this,
             &VMLaunchersViewController::onAddCanceled );
}

void VMLaunchersViewController::bindEditDialog( VMLauncherDialogController* dialogController )
{
    connect( dialogController,
             &VMLauncherDialogController::accepted,
             this,
             &VMLaunchersViewController::onEditAccepted );

    connect( dialogController,
             &VMLauncherDialogController::canceled,
             this,
             &VMLaunchersViewController::onEditCanceled );
}

void VMLaunchersViewController::unbindAddDialog( VMLauncherDialogController* dialogController )
{
    disconnect( dialogController,
                &VMLauncherDialogController::accepted,
                this,
                &VMLaunchersViewController::onAddAccepted );

    disconnect( dialogController,
                &VMLauncherDialogController::canceled,
                this,
                &VMLaunchersViewController::onAddCanceled );
}

void VMLaunchersViewController::unbindEditDialog( VMLauncherDialogController* dialogController )
{
    disconnect( dialogController,
                &VMLauncherDialogController::accepted,
                this,
                &VMLaunchersViewController::onEditAccepted );

    disconnect( dialogController,
                &VMLauncherDialogController::canceled,
                this,
                &VMLaunchersViewController::onEditCanceled );
}

void VMLaunchersViewController::showInfo( const QString& info )
{
    QMessageBox::information( _view, kInfoTitle, info );
}

QSharedPointer<VMLauncher> VMLaunchersViewController::currentLauncher()
{
    if( !_model )
    {
        showInfo( kAccountNotSelected );
        return Q_NULLPTR;
    }

    const auto& launchers = _model->launchers();
    const auto size = launchers.size();

    if( !size )
    {
        showInfo( kNoLaunchers );
        return Q_NULLPTR;
    }

    if( _selectedLauncherIndex < 0 || _selectedLauncherIndex >= size )
    {
        showInfo( kLauncherNotSelected );
        return Q_NULLPTR;
    }

    return launchers[_selectedLauncherIndex];
}

} // namespace PVE
