/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "proxmox-json-parser.h"

#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>
#include <QFile>
#include <QDir>

#include "errors.h"
#include "global.h"

namespace {

const QString kData = "data";
const QString kToken = "CSRFPreventionToken";
const QString kTicket = "ticket";
const QString kUsername = "username";

const QString kNode = "node";
const QString kVmID = "vmid";
const QString kName = "name";
const QString kStatus = "status";
const QString kType = "type";
const QString kRealm = "realm";
const QString kComment = "comment";

const QString kIsNull = "Json is null";
const QString kIsNotAnObject = "Json is not an object";
const QString kDataNotFound = "Data entry not found";
const QString kDataIsNotAnObject = "Data entry is not an object";
const QString kDataIsNotAnArray = "Data entry is not an array";
const QString kDataIsNotAString = "Data entry is not a string";
const QString kTicketNotFound = "Ticket entry not found";
const QString kTokenNotFound = "Token entry not found";
const QString kUsernameNotFound = "Username entry not found";
const QString kVirtualMachineIsNotAnObject = "Virtual machine json is not an object";
const QString kVmIDNotFound = "Vmid entry not found";
const QString kNodeIsNotAnObject = "Node json is not an object";
const QString kNodeNotFound = "Node entry not found";
const QString kStatusNotFound = "Status entry not found";
const QString kTypeNotFound = "Type entry not found";
const QString kRealmNotFound = "Realm entry not found";
const QString kNameNotFound = "Name entry not found";

const QString kJsonParsingErrorMessage = "Could not parse response json: ";

const QString kSpiceProxyFileNotOpened = "Could not open spice-proxy settings file";

const QString kVirtViewerGroup = "virt-viewer";

const QString kSpiceProxyFilePath = QDir::homePath()
                                    + QDir::separator()
                                    + "spiceProxySettings";

QJsonDocument getJson( QNetworkReply* reply )
{
    const auto error = reply->error();

    if( error != QNetworkReply::NoError )
    {
        throw PVE::NetworkException( reply->errorString() );
        if( error <= QNetworkReply::UnknownNetworkError )
        {

        }

        if( error <= QNetworkReply::UnknownProxyError )
        {

        }

        if( error <= QNetworkReply::UnknownContentError )
        {

        }

        if( error <= QNetworkReply::ProtocolFailure )
        {

        }

        if( error <= QNetworkReply::UnknownServerError )
        {

        }
    }

    QJsonParseError parsingError;
    const auto json = QJsonDocument::fromJson( reply->readAll(),
                                               &parsingError );

    if( parsingError.error != QJsonParseError::NoError )
    {
       throw PVE::JsonParsingException( parsingError.errorString() );
    }

    return json;
}

QJsonValue getDataEntry( const QJsonDocument& json )
{
    if( json.isNull() )
    {
        throw PVE::DataEntryParsingException( kIsNull );
    }

    if( !json.isObject() )
    {
        throw PVE::DataEntryParsingException( kIsNotAnObject );
    }

    const auto jsonObject = json.object();

    if( !jsonObject.contains( kData ) )
    {
        throw PVE::DataEntryParsingException( kDataNotFound );
    }

    return jsonObject[kData];
}

QJsonArray getDataEntryAsArray( const QJsonDocument& json )
{
    const auto dataValue = getDataEntry( json );

    if( !dataValue.isArray() )
    {
        throw PVE::DataEntryParsingException( kDataIsNotAnArray );
    }

    return dataValue.toArray();
}

QJsonObject getDataEntryAsObject( const QJsonDocument& json )
{
    const auto dataValue = getDataEntry( json );

    if( !dataValue.isObject() )
    {
        throw PVE::DataEntryParsingException( kDataIsNotAnObject );
    }

    return dataValue.toObject();
}

QString makeSettingsFromJsonObject( const QJsonObject& object )
{
    QFile spiceProxyFile( kSpiceProxyFilePath );

    if( !spiceProxyFile.open( QFile::Text | QFile::WriteOnly ) )
    {
        throw PVE::VirtualMachineParsingException( kSpiceProxyFileNotOpened );
    }

    {
        QTextStream out( &spiceProxyFile );

        out << '[' << kVirtViewerGroup << ']' << endl;

        const auto keys = object.keys();

        for( const auto& key : keys )
        {
            const auto value = object[key];

            if( value.isString() )
            {
                out << key << '=' << value.toString() << endl;
            }
            else
            {
                out << key << '=' << value.toInt() << endl;
            }
        }
    }

    return kSpiceProxyFilePath;
}

} // namespace

namespace PVE {
namespace ProxmoxJsonParser {

TokenData parseTokenResponse( QNetworkReply* reply )
{
    const auto dataObject = getDataEntryAsObject( getJson( reply ) );

    if( !dataObject.contains( kTicket ) )
    {
        throw TokenParsingException( kTicketNotFound );
    }

    if( !dataObject.contains( kToken ) )
    {
        throw TokenParsingException( kTokenNotFound );
    }

    if( !dataObject.contains( kUsername ) )
    {
        throw TokenParsingException( kUsernameNotFound );
    }

    return { dataObject[kUsername].toString(),
             dataObject[kTicket].toString(),
             dataObject[kToken].toString() };
}

QVector<VirtualMachineInfo> parseVirtualMachinesResponse( QNetworkReply* reply )
{
    const auto vms = getDataEntryAsArray( getJson( reply ) );

    QVector<VirtualMachineInfo> result;
    result.reserve( vms.size() );

    for( const auto& vm : vms )
    {
        if( !vm.isObject() )
        {
            throw VirtualMachineParsingException( kVirtualMachineIsNotAnObject );
        }

        const auto vmObject = vm.toObject();

        if( !vmObject.contains( kVmID ) )
        {
            throw VirtualMachineParsingException( kVmIDNotFound );
        }

        if( !vmObject.contains( kName ) )
        {
            throw VirtualMachineParsingException( kNameNotFound );
        }

        result.push_back( VirtualMachineInfo{ vmObject[kVmID].toInt(),
                                              vmObject[kName].toString() } );
    }

    return result;
}

QVector<QString> parseNodesResponse( QNetworkReply* reply )
{
    const auto nodes = getDataEntryAsArray( getJson( reply ) );

    QVector<QString> result;
    result.reserve( nodes.size() );

    for( const auto& node : nodes )
    {
        if( !node.isObject() )
        {
            throw NodesParsingException( kNodeIsNotAnObject );
        }

        const auto nodeObject = node.toObject();

        if( !nodeObject.contains( kNode ) )
        {
            throw NodesParsingException( kNodeNotFound );
        }

        result.push_back( nodeObject[kNode].toString() );
    }

    return result;
}

VirtualMachine::Status
parseVirtualMachineCurrentStatusResponse( QNetworkReply* reply )
{
    const auto dataObject = getDataEntryAsObject( getJson( reply ) );

    if( !dataObject.contains( kStatus ) )
    {
        throw VirtualMachineParsingException( kStatusNotFound );
    }

    const auto string = dataObject[kStatus].toString();
    return Converter::virtualMachineStatusFromString( string );
}

QString parseGetSpiceProxyResponse( QNetworkReply* reply )
{
    const auto dataObject = getDataEntryAsObject( getJson( reply ) );

    return makeSettingsFromJsonObject( dataObject );
}

QString parseVirtualMachineSetStatusResponse( QNetworkReply* reply )
{
    const auto dataValue = getDataEntry( getJson( reply ) );

    if( !dataValue.isString() )
    {
        throw DataEntryParsingException( kDataIsNotAString );
    }

    return dataValue.toString();
}

QVector<QSharedPointer<Domain>> parseDomainsResponse( QNetworkReply* reply )
{
    const auto domains = getDataEntryAsArray( getJson( reply ) );

    QVector<QSharedPointer<Domain>> result;
    result.reserve( domains.size() );

    for( const auto& domain : domains )
    {
        if( !domain.isObject() )
        {
            throw DomainParsingException( kDataIsNotAnObject );
        }

        const auto domainObject = domain.toObject();

        if( !domainObject.contains( kType ) )
        {
            throw DomainParsingException( kTypeNotFound );
        }

        if( !domainObject.contains( kRealm ) )
        {
            throw DomainParsingException( kRealmNotFound );
        }

        const auto typeString = domainObject[kType].toString();
        const auto type = Converter::domainTypeFromString( typeString );
        const auto realm = domainObject[kRealm].toString();
        const auto comment = domainObject[kComment].toString();

        result.push_back( QSharedPointer<Domain>::create( type, realm, comment ) );
    }

    return result;
}

} // namespace ProxmoxJsonParser
} // namespace PVE
