/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "proxmox-node.h"

#include "proxmox-json-parser.h"
#include "errors.h"

namespace {

const QString kVirtualMachinesEndpointFormat = "/nodes/%1/qemu";

} // namespace

namespace PVE {

Node::Node( const QSharedPointer<ProxmoxRestClient>& restClient,
            const QString& id )
    : _restClient( restClient ),
      _id( id ),
      _virtualMachines(),
      _virtualMachinesFetchReply( Q_NULLPTR )
{
    connect( _restClient.data(),
             &ProxmoxRestClient::finished,
             this,
             &Node::onReplyFinished );
}

void Node::fetchVirtualMachines()
{
    if( _virtualMachinesFetchReply )
    {
        return;
    }

    const auto url = QString( kVirtualMachinesEndpointFormat ).arg( _id );

    _virtualMachinesFetchReply = _restClient->get( url );
}

void Node::onReplyFinished( QNetworkReply* reply )
{
    if( !_virtualMachinesFetchReply
            || _virtualMachinesFetchReply != reply )
    {
        return;
    }

    try
    {
        _virtualMachines = processVirtualMachinesFetchReply( reply );
        emit virtualMachinesFetched( _virtualMachines );
    }
    catch( const std::exception& ex )
    {
        emit errorOccurred( ex.what() );
    }

    _virtualMachinesFetchReply = Q_NULLPTR;
    reply->deleteLater();
}

QVector<QSharedPointer<VirtualMachine>>
Node::processVirtualMachinesFetchReply( QNetworkReply* reply )
{
    try
    {
        const auto virtualMachineInfos = ProxmoxJsonParser::parseVirtualMachinesResponse( reply );

        QVector<QSharedPointer<VirtualMachine>> result;
        result.reserve( virtualMachineInfos.size() );

        for( const auto& info : virtualMachineInfos )
        {
            result.push_back( QSharedPointer<VirtualMachine>::create( _restClient,
                                                                      info.vmid,
                                                                      _id,
                                                                      info.name ) );
        }

        return result;
    }
    catch( const VirtualMachineParsingException& ex )
    {
        throw NodeException( ex.what() );
    }
}

} // namespace PVE
