/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "proxmox-rest-client.h"

#include <QNetworkCookie>

namespace {

const QString kProxmoxAPIInputDataMIMEType =
        "application/x-www-form-urlencoded";

const QString kAuthCookie = "PVEAuthCookie";
const QString kAuthHeader = "CSRFPreventionToken";

const QString kJsonAPIPath = "/api2/json";

QNetworkRequest makeRequest( const QUrl& url )
{
    QNetworkRequest request( url );

    request.setHeader( QNetworkRequest::ContentTypeHeader,
                       kProxmoxAPIInputDataMIMEType );

    request.setSslConfiguration( QSslConfiguration::defaultConfiguration() );

    return request;
}

void addAuthCookie( QNetworkRequest& request,
                    const QSharedPointer<PVE::User>& user )
{
    const auto cookie = QNetworkCookie( kAuthCookie.toUtf8(),
                                        user->ticket().toUtf8() );

    request.setHeader( QNetworkRequest::CookieHeader,
                       QVariant::fromValue( cookie ) );
}

} // namespace

namespace PVE {

ProxmoxRestClient::ProxmoxRestClient( const QUrl& baseURL )
    : _baseURL( baseURL ),
      _user(),
      _networkManager(),
      _ignoreSslErrors( false )
{
    connect( &_networkManager,
             &QNetworkAccessManager::finished,
             this,
             &ProxmoxRestClient::finished );

    connect( &_networkManager,
             &QNetworkAccessManager::sslErrors,
             this,
             &ProxmoxRestClient::onSslErrors );

    _networkManager.setCookieJar( Q_NULLPTR );
}

QNetworkReply* ProxmoxRestClient::get( const QString& endpoint )
{
    auto url = _baseURL;
    url.setPath( kJsonAPIPath + endpoint );

    auto request = makeRequest( url );

    if( const auto user = _user )
    {
        addAuthCookie( request, user );
    }

    return _networkManager.get( request );
}

QNetworkReply* ProxmoxRestClient::post( const QString& endpoint,
                                        const QByteArray& data )
{
    auto url = _baseURL;
    url.setPath( kJsonAPIPath + endpoint );

    auto request = makeRequest( url );
    request.setHeader( QNetworkRequest::ContentLengthHeader,
                       QByteArray::number( data.size() ) );

    if( const auto user = _user )
    {
        addAuthCookie( request, user );

        request.setRawHeader( kAuthHeader.toUtf8(), user->token().toUtf8() );
    }

    return _networkManager.post( request, data );
}

void ProxmoxRestClient::onSslErrors( QNetworkReply* reply,
                                     const QList<QSslError>& errors )
{
    if( _ignoreSslErrors )
    {
        reply->ignoreSslErrors();
    }
}

} // namespace PVE
