/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "proxmox-server.h"

#include <QUrlQuery>
#include <QSharedPointer>

#include "proxmox-json-parser.h"

namespace {

const QString kTicketEndpoint = "/access/ticket";
const QString kUsername = "username";
const QString kPassword = "password";

const QString kNodesEndpoint = "/nodes";
const QString kDomainsEndpoint = "/access/domains";

const QString kLoginNetworkErrorMessage = "Login request failed: ";
const QString kLoginJsonParsingErrorMessage = "Could not parse login response json";
const QString kLoginJsonBadFormatMessage = "Login response json bad format: ";
const QString kLoginInvalidURLMessage = "Login URL is invalid: ";

const QString kNodesJsonBadFormatMessage = "Nodes response json bad format: ";

QSharedPointer<PVE::User> processLoginReply( QNetworkReply* reply )
{
    try
    {
        const auto tokenResponseData = PVE::ProxmoxJsonParser::parseTokenResponse( reply );

        return QSharedPointer<PVE::User>::create( tokenResponseData.username,
                                                  tokenResponseData.ticket,
                                                  tokenResponseData.token );
    }
    catch( const PVE::TokenParsingException& ex )
    {
        throw PVE::ProxmoxClientException( kLoginJsonBadFormatMessage
                                           + ex.what() );
    }
}

} // namespace

namespace PVE {

Server::Server( const QUrl& serverURL )
    : _restClient( QSharedPointer<ProxmoxRestClient>::create( serverURL ) ),
      _loginReply( Q_NULLPTR ),
      _fetchNodesReply( Q_NULLPTR ),
      _fetchDomains( Q_NULLPTR ),
      _fetchStatus( Q_NULLPTR ),
      _user(),
      _nodes(),
      _status( Status::DOWN )
{
    connect( _restClient.data(),
             &ProxmoxRestClient::finished,
             this,
             &Server::onReplyFinished );
}

void Server::login( const QString& username, 
                           const QString& password )
{
    if( _loginReply )
    {
        return;
    }

    QUrlQuery postData;
    postData.addQueryItem( kUsername, QUrl::toPercentEncoding( username ) );
    postData.addQueryItem( kPassword, QUrl::toPercentEncoding( password ) );

    const auto data = postData.query().toUtf8();

    _loginReply = _restClient->post( kTicketEndpoint, data );
}

void Server::fetchNodes()
{
    if( !_user || _fetchNodesReply )
    {
        return;
    }

    _fetchNodesReply = _restClient->get( kNodesEndpoint );
}

void Server::fetchDomains()
{
    if( _fetchDomains )
    {
        return;
    }

    _fetchDomains = _restClient->get( kDomainsEndpoint );
}

void Server::fetchStatus()
{
    if( _fetchStatus )
    {
        return;
    }

    _fetchStatus = _restClient->get( kDomainsEndpoint );
}

void Server::onReplyFinished( QNetworkReply* reply )
{
    if( _loginReply && _loginReply == reply  )
    {
        try
        {
            _user = processLoginReply( reply );
            _restClient->setUser( _user );

            emit loginSucceeded();
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _loginReply = Q_NULLPTR;
        reply->deleteLater();

        return;
    }

    if( _fetchNodesReply && _fetchNodesReply == reply )
    {
        try
        {
            _nodes = processFetchNodesReply( reply );
            emit nodesFetched( _nodes );
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _fetchNodesReply = Q_NULLPTR;
        reply->deleteLater();

        return;
    }

    if( _fetchDomains && _fetchDomains == reply )
    {
        try
        {
            emit domainsFetched( ProxmoxJsonParser::parseDomainsResponse( reply ) );
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _fetchDomains = Q_NULLPTR;
        reply->deleteLater();

        return;
    }

    if( _fetchStatus && _fetchStatus == reply )
    {
        auto status = Server::Status::DOWN;

        try
        {            
            if( ProxmoxJsonParser::parseDomainsResponse( reply ).size() )
            {
                status = Server::Status::UP;
            }
        }
        catch( const std::exception& )
        {
            // HOST IS DOWN
        }

        setStatus( status );
        emit statusFetched( status );

        _fetchStatus = Q_NULLPTR;
        reply->deleteLater();

        return;
    }
}

QVector<QSharedPointer<Node>> Server::processFetchNodesReply( QNetworkReply* reply )
{
    try
    {
        const auto nodeIDs = ProxmoxJsonParser::parseNodesResponse( reply );

        QVector<QSharedPointer<Node>> result;
        result.reserve( nodeIDs.size() );

        for( const auto id : nodeIDs )
        {
            result.push_back( QSharedPointer<Node>::create( _restClient, id ) );
        }

        return result;
    }
    catch( const NodesParsingException& ex )
    {
        throw ProxmoxClientException( kNodesJsonBadFormatMessage
                                      + ex.what() );
    }
}

} // namespace PVE
