/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "proxmox-virtual-machine.h"

#include <QUrlQuery>

#include "proxmox-json-parser.h"

namespace {

const QString kStateEndpointFormat = "/nodes/%1/qemu/%2/status/current";
const QString kStartEndpointFormat = "/nodes/%1/qemu/%2/status/start";
const QString kSpiceProxyEndpointFormat = "/nodes/%1/qemu/%2/spiceproxy";
const QString kProxy = "proxy";

const QString kInvalidSetStatusRespose = "Could not set status: invalid response";

const int kTimeoutInterval = 1000;

} // namespace

namespace PVE {

VirtualMachine::VirtualMachine( const QSharedPointer<ProxmoxRestClient>& restClient,
                                int vmID,
                                const QString& nodeID,
                                const QString& name )
    : _restClient( restClient ),
      _vmID( vmID ),
      _nodeID( nodeID ),
      _name( name ),
      _status( Status::Stopped ),
      _timer(),
      _fetchStatusReply( Q_NULLPTR ),
      _fetchSpiceProxyReply( Q_NULLPTR ),
      _startReply( Q_NULLPTR )
{
    connect( _restClient.data(),
             &ProxmoxRestClient::finished,
             this,
             &VirtualMachine::onReplyFinished );

    connect( &_timer,
             &QTimer::timeout,
             this,
             &VirtualMachine::onTimeout );

    _timer.setInterval( kTimeoutInterval );
    _timer.start();
}

void VirtualMachine::fetchStatus()
{
    if( _fetchStatusReply )
    {
        return;
    }

    const auto url = QString( kStateEndpointFormat )
                     .arg( _nodeID, QString::number( _vmID ) );

    _fetchStatusReply = _restClient->get( url );
}

void VirtualMachine::fetchSpiceProxy()
{
    if( _fetchSpiceProxyReply )
    {
        return;
    }

    const auto url = QString( kSpiceProxyEndpointFormat )
                     .arg( _nodeID, QString::number( _vmID ) );

    const auto host = _restClient->url().host();

    QUrlQuery postData;
    postData.addQueryItem( kProxy, QUrl::toPercentEncoding( host ) );

    const auto data = postData.query().toUtf8();

    _fetchSpiceProxyReply = _restClient->post( url, data );
}

void VirtualMachine::start()
{
    if( _startReply )
    {
        return;
    }

    const auto url = QString( kStartEndpointFormat )
                     .arg( _nodeID, QString::number( _vmID ) );

    _startReply = _restClient->post( url, QByteArray() );
}

void VirtualMachine::onReplyFinished( QNetworkReply* reply )
{
    if( _fetchStatusReply && _fetchStatusReply == reply )
    {
        try
        {
            const auto newStatus = ProxmoxJsonParser::parseVirtualMachineCurrentStatusResponse( reply );
            emit statusFetched( newStatus );

            if( _status != newStatus )
            {
                _status = newStatus;
                emit changed( this );
            }
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _fetchStatusReply = Q_NULLPTR;
        reply->deleteLater();
        return;
    }

    if( _fetchSpiceProxyReply && _fetchSpiceProxyReply == reply )
    {
        try
        {
            emit spiceProxyFetched( ProxmoxJsonParser::parseGetSpiceProxyResponse( reply ) );
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _fetchSpiceProxyReply = Q_NULLPTR;
        reply->deleteLater();
        return;
    }

    if( _startReply && _startReply == reply )
    {
        try
        {
            const auto response = ProxmoxJsonParser::parseVirtualMachineSetStatusResponse( reply );

            if( response.isEmpty() )
            {
                emit errorOccurred( kInvalidSetStatusRespose );
            }
            else
            {
                emit statusSet();
            }
        }
        catch( const std::exception& ex )
        {
            emit errorOccurred( ex.what() );
        }

        _startReply = Q_NULLPTR;
        reply->deleteLater();
        return;
    }
}

void VirtualMachine::onTimeout()
{
    fetchStatus();
}

} // namespace PVE
