/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settings-manager.h"

#include <QJsonDocument>
#include <QFile>
#include <QSettings>
#include <QTextStream>
#include <QDir>
#include <QStandardPaths>
#include <QApplication>

#include "errors.h"
#include "settings-parser.h"

#ifdef USE_PASSWORD_STORAGE
#include "keychain.h"
#endif

namespace {

const QString kSettingsFolderCreationFailed = "Could not create settings folder";
const QString kSettingsFileReadFailed = "Could not open settings file for reading";
const QString kSettingsJsonParseFailed = "Could not parse settings json";

const QString kSettingsExtention = ".conf";

void sortAccounts( QVector<QSharedPointer<PVE::Account>>& accounts )
{
    auto rule = []( const QSharedPointer<PVE::Account>& lhs,
                    const QSharedPointer<PVE::Account>& rhs )
    {
        return lhs->name() < rhs->name();
    };

    std::sort( accounts.begin(), accounts.end(), rule );
}

} // namespace

namespace PVE {

SettingsManager::SettingsManager()
    : _settings()
{
    const auto settingsDirPath = QStandardPaths::writableLocation( QStandardPaths::ConfigLocation )
                                 + QDir::separator()
                                 + QApplication::organizationName();
    QDir settingsDir;

    if( !settingsDir.exists( settingsDirPath ) )
    {
        if( !settingsDir.mkpath( settingsDirPath ) )
        {
            throw SettingsManagerException( kSettingsFolderCreationFailed );
        }
    }

    const auto settingsFilePath = settingsDirPath
                                  + QDir::separator()
                                  + QApplication::applicationName()
                                  + kSettingsExtention;

    QFile settingsFile( settingsFilePath );

    if( !settingsFile.open( QFile::Text | QFile::ReadWrite ) )
    {
        throw SettingsManagerException( kSettingsFileReadFailed );
    }

    if( settingsFile.size() )
    {
        QJsonParseError parsingError;
        const auto json = QJsonDocument::fromJson( settingsFile.readAll(),
                                                   &parsingError );

        if( parsingError.error != QJsonParseError::NoError )
        {
           throw SettingsManagerException( kSettingsJsonParseFailed );
        }

        _settings = SettingsParser::fromJson( json.object() );
    }
}

SettingsManager::~SettingsManager()
{
    const auto settingsDirPath = QStandardPaths::writableLocation( QStandardPaths::ConfigLocation )
                                 + QDir::separator()
                                 + QApplication::organizationName();
    QDir settingsDir;

    if( !settingsDir.exists( settingsDirPath ) )
    {
        if( !settingsDir.mkpath( settingsDirPath ) )
        {
            return;
        }
    }

    const auto settingsFilePath = settingsDirPath
                                  + QDir::separator()
                                  + QApplication::applicationName()
                                  + kSettingsExtention;

    QFile settingsFile( settingsFilePath );

    if( !settingsFile.open( QFile::Text | QFile::WriteOnly ) )
    {
        return;
    }

    QTextStream out( &settingsFile );
    out << QJsonDocument( SettingsParser::toJson( _settings ) ).toJson();
}

SettingsManager& SettingsManager::get()
{
    static SettingsManager manager;
    return manager;
}

QSharedPointer<Account> SettingsManager::lastAccount() const
{
    return _settings.lastAccount;
}

void SettingsManager::setLastAccount( const QSharedPointer<Account>& account )
{
    _settings.lastAccount = account;
}

const QVector<QSharedPointer<Account>>& SettingsManager::accounts() const
{
    return _settings.accounts;
}

void SettingsManager::addAccount( const QSharedPointer<Account>& account )
{
    _settings.accounts.push_back( account );
    sortAccounts( _settings.accounts );
#ifdef USE_PASSWORD_STORAGE
    const auto username = QString( "%1@%2" )
                          .arg( account->user() )
                          .arg( account->realm() );

    SyncKeychain::setUserPassword( username,
                                   account->server()->url().toString(),
                                   account->password() );
#endif
}

void SettingsManager::removeAccount( const QSharedPointer<Account>& account )
{
    _settings.accounts.removeAll( account );
    sortAccounts( _settings.accounts );
}

} // namespace PVE
