/*
This file is part of PVE Spice Client.

PVE Spice Client is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PVE Spice Client is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PVE Spice Client.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settings-parser.h"

#include "config.h"

#ifdef USE_PASSWORD_STORAGE
#include "keychain.h"
#endif

#include <QJsonArray>

namespace {

const QString kLastAccount = "lastAccount";
const QString kAccounts = "accounts";
const QString kName = "name";
const QString kUser = "user";
const QString kRealm = "realm";
const QString kServer = "server";
const QString kUrl = "url";
const QString kIgnoreSslErrors = "ignoreSslErrors";
const QString kLaunchers = "launchers";
const QString kId = "id";
const QString kArguments = "arguments";
const QString kFullscreen = "fullscreen";
const QString kLastLauncher = "lastLauncher";
const QString kAccountsUpdatingInterval = "accountsUpdatingInterval";
const QString kLaunchersUpdatingInterval = "launchersUpdatingInterval";

const QString kEntryNotFoundFormat = "Entry not found: %1";

} // namespace

namespace PVE {
namespace SettingsParser {

#ifdef USE_PASSWORD_STORAGE
QString getPasswordFromKeychain( const QString& user,
                                 const QString& realm,
                                 const QString& url )
{
    return SyncKeychain::getUserPassword( QString( "%1@%2" )
                                          .arg( user )
                                          .arg( realm ),
                                          url );
}
#endif

VMLaunchArguments argumentsFromJson( const QJsonObject& object )
{
    if( !object.contains( kFullscreen ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kFullscreen ) );
    }

    return { object[kFullscreen].toBool() };
}

QSharedPointer<VMLauncher> launcherFromJson( const QJsonObject& object )
{
    if( !object.contains( kName ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kName ) );
    }

    if( !object.contains( kId ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kId ) );
    }

    if( !object.contains( kArguments ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kArguments ) );
    }

    const auto argumentsObject = object[kArguments].toObject();

    return QSharedPointer<VMLauncher>::create( object[kName].toString(),
                                               object[kId].toInt(),
                                               argumentsFromJson( argumentsObject ) );
}

QSharedPointer<Server> serverFromJson( const QJsonObject& object )
{
    if( !object.contains( kUrl ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kUrl ) );
    }

    if( !object.contains( kIgnoreSslErrors ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kIgnoreSslErrors ) );
    }

    const auto server = QSharedPointer<Server>::create( object[kUrl].toString() );
    server->setIgnoreSslErrors( object[kIgnoreSslErrors].toBool() );

    return server;
}

QSharedPointer<Account> accountFromJson( const QJsonObject& object )
{
    if( !object.contains( kName ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kName ) );
    }

    if( !object.contains( kUser ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kUser ) );
    }

    if( !object.contains( kRealm ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kRealm ) );
    }

    if( !object.contains( kServer ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kServer ) );
    }

    if( !object.contains( kLaunchers ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kLaunchers ) );
    }

    const auto name = object[kName].toString();
    const auto user = object[kUser].toString();
    const auto realm = object[kRealm].toString();
    const auto server = serverFromJson( object[kServer].toObject() );

    if( !object[kLaunchers].isArray() )
    {
        throw SettingsManagerException( "Launchers entry is not an array" );
    }

    const auto launchersArray = object[kLaunchers].toArray();
    QVector<QSharedPointer<VMLauncher>> launchers;
    launchers.reserve( launchersArray.size() );

    for( const auto& launcherValue : launchersArray )
    {
        launchers.push_back( launcherFromJson( launcherValue.toObject() ) );
    }

    auto rule = []( const QSharedPointer<VMLauncher>& lhs,
                    const QSharedPointer<VMLauncher>& rhs )
    {
        return lhs->vmid() < rhs->vmid();
    };

    std::sort( launchers.begin(), launchers.end(), rule );

    QString password;

#ifdef USE_PASSWORD_STORAGE
    password =  getPasswordFromKeychain( user,
                                         realm,
                                         server->url().toString() );
#endif

    return QSharedPointer<Account>::create( name,
                                            user,
                                            realm,
                                            password,
                                            server,
                                            launchers );
}

Settings fromJson( const QJsonObject& object )
{
    if( !object.contains( kLastAccount ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kLastAccount ) );
    }

    if( !object.contains( kAccounts ) )
    {
        throw SettingsManagerException( QString( kEntryNotFoundFormat )
                                        .arg( kAccounts ) );
    }

    if( !object[kAccounts].isArray() )
    {
        throw SettingsManagerException( "Accounts entry is not an array" );
    }

    const auto accountsArray = object[kAccounts].toArray();
    const auto lastAccountName = object[kLastAccount].toString();
    QSharedPointer<Account> lastAccount;
    QVector<QSharedPointer<Account>> accounts;

    for( const auto& accountValue : accountsArray )
    {
        const auto accountObject = accountValue.toObject();

        const auto account = accountFromJson( accountObject );

        if( account->name() == lastAccountName )
        {
            lastAccount = account;
        }

        if( accountObject.contains( kLastLauncher ) )
        {
            const auto lastLauncherID = accountObject[kLastLauncher].toInt();

            auto rule = [lastLauncherID]( const QSharedPointer<VMLauncher>& launcher )
            {
                return launcher->vmid() == lastLauncherID;
            };

            const auto launchers = account->launchers();
            const auto found = std::find_if( launchers.begin(),
                                             launchers.end(),
                                             rule );
            if( found != launchers.end() )
            {
                account->setLastLauncher( *found );
            }
        }

        accounts.push_back( account );
    }

    auto rule = []( const QSharedPointer<PVE::Account>& lhs,
                    const QSharedPointer<PVE::Account>& rhs )
    {
        return lhs->name() < rhs->name();
    };

    std::sort( accounts.begin(), accounts.end(), rule );

    Settings settings( lastAccount, accounts );

    if( object.contains( kAccountsUpdatingInterval ) )
    {
        settings.accountsUpdatingInterval = object[kAccountsUpdatingInterval].toInt();
    }

    if( object.contains( kLaunchersUpdatingInterval ) )
    {
        settings.launchersUpdatingInterval = object[kLaunchersUpdatingInterval].toInt();
    }

    return settings;
}

QJsonObject serverToJson( const QSharedPointer<Server>& server )
{
    QJsonObject json;

    json[kUrl] = server->url().toString();
    json[kIgnoreSslErrors] = server->ignoreSslErrors();

    return json;
}

QJsonObject argumentsToJson( const VMLaunchArguments& arguments )
{
    QJsonObject json;

    json[kFullscreen] = arguments.fullscreen;

    return json;
}

QJsonObject launcherToJson( const QSharedPointer<VMLauncher>& launcher )
{
    QJsonObject json;

    json[kName] = launcher->name();
    json[kId] = launcher->vmid();
    json[kArguments] = argumentsToJson( launcher->arguments() );

    return json;
}

QJsonObject accountToJson( const QSharedPointer<Account>& account )
{
    QJsonObject json;

    json[kName] = account->name();
    json[kUser] = account->user();
    json[kRealm] = account->realm();
    json[kServer] = serverToJson( account->server() );

    QJsonArray launchersArray;
    const auto launchers = account->launchers();

    for( const auto& launcher : launchers )
    {
        launchersArray.push_back( launcherToJson( launcher ) );
    }

    json[kLaunchers] = launchersArray;

    if( const auto lastLauncher = account->lastLauncher() )
    {
        json[kLastLauncher] = lastLauncher->vmid();
    }

    return json;
}

QJsonObject toJson( const Settings& settings )
{
    QJsonObject json;

    json[kLastAccount] = settings.lastAccount ? settings.lastAccount->name()
                                              : QString();
    QJsonArray accountsArray;
    const auto& accounts = settings.accounts;

    for( const auto& account : accounts )
    {
        accountsArray.push_back( accountToJson( account ) );
    }

    json[kAccounts] = accountsArray;
    json[kAccountsUpdatingInterval] = settings.accountsUpdatingInterval;
    json[kLaunchersUpdatingInterval] = settings.launchersUpdatingInterval;

    return json;
}

} // namespace SettingsParser
} // namespace PVE
